# CompanyApplicationObject.pm - The Object Class that provides a CompanyApplicationObject Object
# Created by James A. Pattie, 11/07/2000.

# Copyright (c) 2000-2003 Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::Objects::CompanyApplicationObject;
use strict;
use overload '==' => \&objsEqual,
             '!=' => \&objsNotEqual,
             '""' => \&printStr;
use Portal::Base;
use Portal::Data::Variables;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.08';

=head1 NAME

CompanyApplicationObject - Object used to build a CompanyApplicationObject Object Class.

=head1 SYNOPSIS

  use Portal::Objects::CompanyApplicationObject;
  my $obj = Portal::Objects::CompanyApplicationObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

CompanyApplicationObject is a CompanyApplicationObject class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Objects::CompanyApplicationObject module.
 See Portal::Base(3) for a listing of required arguments.

 You now have to use the populate() method to assign the 
 CompanyApplicationObject parameters and have them be validated.  
 new() just instantiates the object so that it can be used by the
 Portal::Forms::Apps module to manage Applications with.
 
 The HTMLObject::Forms data structures will be defined for
 the following cases:
 
 assignment - allow an admin to assign/un-assign the app
  to users of a specific company.  This form has a
  javascript code block defined which needs to be output
  into the javascript section of the HTMLObject document
  you display the form in.  (It is not an include file yet).
  
 serverInfo - allow an admin to edit the server info for
  the specified company_app_tb entry.
  
 purchaseApp - used to let the Company admin purchase the
  app and do initial server selection, etc.
  
 sellAppLicenses - allows an admin to sell off spare
  licenses or the app itself.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{variables} = Portal::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{variables}->didErrorOccur)
  {
    $self->setError(errorString => $self->{variables}->errorMessage);

    return $self;
  }
  
  # the templates
  my $indicatorsPhrase = $self->{langObj}->map("DisabledAdminSysadminIndicators");
  $self->{template}->{assignment} = <<"END_OF_CODE";
<form #FORMARGS#>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=appId#
  #FIELD=formSubmitted#
  <table border="0" cellpadding="2" cellspacing="0" class="edit">
    <tr>
      <td align="center" valign="top">#FIELD=users#</td>
    </tr>
    <tr>
      <td align="center">#FIELD=submitForm#</td>
    </tr>
    <tr>
      <td align="center"><span style="font-size: smaller;">$indicatorsPhrase</span></td>
    </tr>
  </table>  
</form>  
END_OF_CODE

  $self->{data}->{assignment} = { "formSubmitted" => { -Type => "hidden", -Value => "1" },
                               "submitForm" => { -Type => "submit", -Value => $self->{langObj}->map("UpdateAppAssignments") },
                               "app" => { -Type => "hidden", -Value => "" },
                               "state" => { -Type => "hidden", -Value => "" },
                               "command" => { -Type => "hidden", -Value => "updateAppAssignments" },
                               "appId" => { -Type => "hidden", -Value => "" },
                               "users" => { -Type => "select-picker", -assignedName => "assignedUsers",
                                 -unassignedName => "unassignedUsers", -assignedLabel => $self->{langObj}->map("Assigned"),
                                 -unassignedLabel => $self->{langObj}->map("UnAssigned"),
                                 -seperator => ",", -assignedValues => "", -unassignedValues => "",
                                 -assignedOptions => undef, -unassignedOptions => undef, class => "edit" },
                             };
                             
  $self->{profile}->{assignment} = { required => [],
                                  optional => [],
                                  constraints => {},
                                };

  my $serverInfoPhrase = $self->{langObj}->map("ServerInfo");
  $self->{template}->{serverInfo} = <<"END_OF_CODE";
<span style="font-size: smaller;">$serverInfoPhrase</span><br />
<form #FORMARGS#>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=appId#
  #FIELD=formSubmitted#
  <table border="0" cellpadding="2" cellspacing="0" class="edit">
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=serverInfo#: </span></td>
      <td align="left">#FIELD=serverInfo#</td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=dbName#: </span></td>
      <td align="left"><span style="font-size: smaller;">#FIELD=dbName#</span></td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=autorun#: #INVALID=autorun#</span></td>
      <td align="left">#FIELD=autorun#</td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=wap#: #INVALID=wap#</span></td>
      <td align="left">#FIELD=wap#</td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=width#: #INVALID=width#</span></td>
      <td align="left">#FIELD=width#</td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=height#: #INVALID=height#</span></td>
      <td align="left">#FIELD=height#</td>
    </tr>
    <tr>
      <td colspan="2" align="center">#FIELD=submitForm#</td>
    </tr>
  </table>  
</form>  
END_OF_CODE

  $self->{data}->{serverInfo} = { "formSubmitted" => { -Type => "hidden", -Value => "1" },
                               "submitForm" => { -Type => "submit", -Value => $self->{langObj}->map("UpdateServerInfo") },
                               "app" => { -Type => "hidden", -Value => "" },
                               "state" => { -Type => "hidden", -Value => "" },
                               "command" => { -Type => "hidden", -Value => "updateServerInfo" },
                               "appId" => { -Type => "hidden", -Value => "" },
                               "serverInfo" => { -Type => "select", -Value => "", -Label => $self->{langObj}->map("Server"), -Options => undef },
                               "dbName" => { -Type => "text", -Value => "", -Label => $self->{langObj}->map("DBName"), -ReadOnly => 1, -ReadOnlyMode => "text" },
                               "autorun" => { -Type => "select", -Value => "", -Label => $self->{langObj}->map("AutoRun"), -Options => $self->{variables}->formBoolean() },
                               "wap" => { -Type => "select", -Value => "", -Label => $self->{langObj}->map("WAPEnabled"), -Options => $self->{variables}->formBoolean() },
                               "width" => { -Type => "text", -Value => "", -Label => $self->{langObj}->map("Width"), size => 4, maxlength => 4 },
                               "height" => { -Type => "text", -Value => "", -Label => $self->{langObj}->map("Height"), size => 4, maxlength => 4 },
                             };
                             
  $self->{profile}->{serverInfo} = { required => [ qw( serverInfo autorun wap width height app state command ) ],
                                  optional => [],
                                  constraints => {},
                                };

  my $purchaseAppPhrase = $self->{langObj}->map("PurchaseApplication");
  my $cancelPurchasePhrase = $self->{langObj}->map("ToCancelClickManageApps");
  $self->{template}->{purchaseApp} = <<"END_OF_CODE";
<span style="font-size: smaller;">$purchaseAppPhrase</span><br />
<form #FORMARGS#>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=appId#
  #FIELD=formSubmitted#
  <table border="0" cellpadding="2" cellspacing="0" class="edit">
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=name#: </span></td>
      <td align="left"><span style="font-size: smaller;">#FIELD=name#</span></td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=description#: </span></td>
      <td align="left"><span style="font-size: smaller;">#FIELD=description#</span></td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=cost#: </span></td>
      <td align="left"><span style="font-size: smaller;">#FIELD=cost#</span></td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=unit#: </span></td>
      <td align="left"><span style="font-size: smaller;">#FIELD=unit#</span></td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=numLicenses#: #INVALID=numLicenses#</span></td>
      <td align="left">#FIELD=numLicenses#</td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=server#: </span></td>
      <td align="left">#FIELD=server#</td>
    </tr>
    <tr>
      <td align="right" valign="top"><span style="font-size: smaller;">#LABEL=dbName#: </span></td>
      <td align="left">#FIELD=dbName#</td>
    </tr>
    <tr>
      <td colspan="2" align="center">#FIELD=submitForm#<br /><span style="font-size: smaller;">$cancelPurchasePhrase</span></td>
    </tr>
  </table>  
</form>  
END_OF_CODE

  $self->{data}->{purchaseApp} = { "formSubmitted" => { -Type => "hidden", -Value => "1" },
                               "submitForm" => { -Type => "submit", -Value => $self->{langObj}->map("Purchase") },
                               "app" => { -Type => "hidden", -Value => "" },
                               "state" => { -Type => "hidden", -Value => "" },
                               "command" => { -Type => "hidden", -Value => "purchaseApp" },
                               "appId" => { -Type => "hidden", -Value => "" },
                               "server" => { -Type => "select", -Value => "", -Label => $self->{langObj}->map("Server"), -Options => undef },
                               "dbName" => { -Type => "text", -Value => "", -Label => $self->{langObj}->map("DBName"), -ReadOnly => 1, -ReadOnlyMode => "text" },
                               "numLicenses" => { -Type => "select", -Value => "", -Label => $self->{langObj}->map("numLicenses"), -Options => undef },
                               "name" => { -Type => "text", -Value => "", -Label => $self->{langObj}->map("Name"), -ReadOnly => 1, -ReadOnlyMode => "text" },
                               "description" => { -Type => "text", -Value => "", -Label => $self->{langObj}->map("Description"), -ReadOnly => 1, -ReadOnlyMode => "text" },
                               "cost" => { -Type => "text", -Value => "", -Label => $self->{langObj}->map("Cost"), -ReadOnly => 1, -ReadOnlyMode => "text" },
                               "unit" => { -Type => "select", -Value => "", -Options => undef, -Label => $self->{langObj}->map("Unit"), -ReadOnly => 1, -ReadOnlyMode => "text" },
                             };
                             
  $self->{profile}->{purchaseApp} = { required => [ qw( server numLicenses dbName app state command ) ],
                                  optional => [],
                                  constraints => {},
                                };

  $self->{template}->{sellAppLicenses} = <<"END_OF_CODE";
<span style="font-size: smaller;">#SELLAPPLICENSESPHRASE#</span><br />
<form #FORMARGS#>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=appId#
  #FIELD=formSubmitted#
  <table border="0" cellpadding="2" cellspacing="0" class="edit">
    <tr>
      <td align="left"><span style="font-size: smaller;">#FIELD=type#</span></td>
    </tr>
    <tr>
      <td align="left"><span style="font-size: smaller;">#FIELD=numLicenses#</span></td>
    </tr>
    <tr>
      <td align="center">#FIELD=submitForm#</td>
    </tr>
  </table>  
</form>  
END_OF_CODE

  $self->{data}->{sellAppLicenses} = { "formSubmitted" => { -Type => "hidden", -Value => "1" },
                               "submitForm" => { -Type => "submit", -Value => $self->{langObj}->map("Sell") },
                               "app" => { -Type => "hidden", -Value => "" },
                               "state" => { -Type => "hidden", -Value => "" },
                               "command" => { -Type => "hidden", -Value => "sellApp" },
                               "appId" => { -Type => "hidden", -Value => "" },
                               "type" => { -Type => "radio", -Value => "licenses", -Options => { values => [ "app", "licenses" ], labels => [ $self->{langObj}->map("Application"), $self->{langObj}->map("Licenses") ] } },
                               "numLicenses" => { -Type => "select", -Value => "", -Options => undef },
                             };
                             
  $self->{profile}->{sellAppLicenses} = { required => [ qw( type appId app state command ) ],
                                  optional => [ qw( numLicenses ) ],
                                  constraints => {},
                                };

  return $self;
}

=item bool populate(appId, companyId, number, server, port, cost, unit, dbName,
dbHost, dbType, dbPort, height, width, autorun, wap)

 Populates the CompanyApplicationObject instance with the
 specified info and validates them via the isValid() method.
 Updates the HTMLObject::Form's data structures.
 
 Returns the true/false result from isValid().
 
 requires:
   appId
   companyId
   number  - the number of licenses (0 = unlimited)
   server  - web server hosting this companies instance of the app
   port    - 80 or 443 - indicates http/https connections.
   cost
   unit
   dbHost
   dbName
   dbType
   dbPort
   height
   width
   autorun
   wap

=cut
sub populate
{
  my $self = shift;
  my %args = ( appId => -1, companyId => -1, number => 1, server => "", port => -1, cost => "0.00", unit => 0, dbName => "", height => "640",
               width => "480", autorun => 0, dbType => "Pg", dbPort => "5432", wap => 0, dbHost => "127.0.0.1", @_ );

  # assign to $self and update the data profiles.
  foreach my $name (qw( appId companyId number server port cost unit dbName height width autorun dbHost dbPort dbType wap ))
  {
    $self->{$name} = $args{$name};
    foreach my $profile (keys %{$self->{data}})
    {
      $self->{data}->{$profile}->{$name}->{-Value} = $args{$name} if (exists $self->{data}->{$profile}->{$name});
    }
  }

  # do validation
  return $self->isValid;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->{appId} < 0)
  {
    $self->invalid("appId", "$self->{appId}");
  }
  if ($self->{companyId} < 0)
  {
    $self->invalid("companyId", "$self->{companyId}");
  }
  if ($self->{number} < 0)  # 0 is valid as it represents unlimited licenses.
  {
    $self->invalid("number", "$self->{number}");
  }
  if (length $self->{server} == 0)
  {
    $self->invalid("server", "$self->{server}");
  }
  if ($self->{port} < 0)
  {
    $self->invalid("port", "$self->{port}");
  }
  if ($self->{cost} !~ /^\d+\.\d{2}$/)
  {
    $self->invalid("cost", "$self->{cost}");
  }
  if (!exists $self->{variables}->{units}->{$self->{unit}})
  {
    $self->invalid("unit", "$self->{unit}");
  }
  if (length $self->{dbName} == 0)
  {
    $self->invalid("dbName", "$self->{dbName}");
  }
  if ($self->{height} < 0 || $self->{height} > 4000)  # I think 4000 is way too much!
  {
    $self->invalid("height", "$self->{height}");
  }
  if ($self->{width} < 0 || $self->{width} > 4000)
  {
    $self->invalid("width", "$self->{width}");
  }
  if ($self->{autorun} !~ /^(1|0)$/)
  {
    $self->invalid("autorun", "$self->{autorun}");
  }
  if (length $self->{dbHost} == 0)
  {
    $self->missing("dbHost");
  }
  if ($self->{dbType} !~ /^(Pg|mysql)$/)
  {
    $self->invalid("dbType", "$self->{dbType}");
  }
  if ($self->{dbPort} !~ /^\d+$/)
  {
    $self->invalid("dbPort", "$self->{dbPort}");
  }
  if ($self->{wap} !~ /^(1|0)$/)
  {
    $self->invalid("wap", "$self->{wap}");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item bool objsEqual(a,b)

 returns 1 if the 2 objects are the same (content wise),
 0 otherwise.
 
=cut
sub objsEqual
{
  my $a = shift;
  my $b = shift;
  
  if (ref($a) ne "Portal::Objects::CompanyApplicationObject")
  {
    die "objsEqual: a = '" . ref($a) . "'";
  }
  if (ref($b) ne "Portal::Objects::CompanyApplicationObject")
  {
    die "objsEqual: b = '" . ref($b) . "'";
  }

  # do number tests.
  foreach my $key (qw(appId companyId number port unit dbPort height width autorun wap))
  {
    return 0 if ($a->{$key} != $b->{$key});
  }
  
  # do string tests
  foreach my $key (qw(server cost dbHost dbName dbType))
  {
    return 0 if ($a->{$key} ne $b->{$key});
  }
  
  return 1;  # assume they are equal if we get this far.  
}

=item bool objsNotEqual(a,b)

 returns 0 if the 2 objects are the same (content wise),
 1 otherwise.
 
=cut
sub objsNotEqual
{
  my $a = shift;
  my $b = shift;
  
  if (ref($a) ne "Portal::Objects::CompanyApplicationObject")
  {
    die "objsNotEqual: a = '" . ref($a) . "'";
  }
  if (ref($b) ne "Portal::Objects::CompanyApplicationObject")
  {
    die "objsNotEqual: b = '" . ref($b) . "'";
  }

  # take advantage of the == overload to do the brunt of our work.
  return (!($a == $b));
}

=item scalar printStr()

 returns the printable version of this object.
 
=cut
sub printStr
{
  my $self = shift;
  my $out = "Portal::Objects::CompanyApplicationObject : ";
  
  foreach my $keys (keys %{$self})
  {
    $out .= ", " if ($out =~ /=/);  # output a , if we have an = in the string.
    $out .= "$keys = '" . $self->{$keys} . "'";
  }
  
  $out .= "\n";
  
  return $out;
}

=back

=cut

1;
__END__

=head1 Exported VARIABLES

  appId     - int
  companyId - int
  number    - int (number of licenses bought)
  server    - text
  port      - numeric(6)
  dbHost    - text
  dbType    - text
  dbPort    - numeric(6)
  cost      - money (without $ and ,'s)
  unit      - int
  dbName    - text
  height    - int
  width     - int
  autorun   - bool
  wap       - bool


=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Application(3), Portal::Base(3)

=cut
