# Session.pm - The Object Class that provides a Session Object
# Created by James A. Pattie, 11/07/2000.

# Copyright (c) 2000-2003 Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::Session;
use strict;
use Portal::Base;
use Portal::Methods;
use Portal::Data::Variables;
use DBIWrapper;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.10';

=head1 NAME

Session - Object used to build a Session Object Class.

=head1 SYNOPSIS

  use Portal::Session;
  my $obj = Portal::Session->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Session is a Session class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbiw, dbType, cookieObj, appName, cookieDomain,
      cookieLife, sessionType, fileDir, lockDir, lockTimeout,
      sessionId)

 Creates a new instance of the Session object.
 See Portal::Base(3) for a listing of required arguments.

 requires: sessionType - Database or File

 if sessionType = Database
  dbiw - DBIWrapper object pointing to Portal Database
  dbType - database type (Postgres, MySQL)
 elsif sessionType = File
  fileDir - name of Directory to store sessions in
  lockDir - Directory to use for lock files
  lockTimeout - the number of seconds to allow lock files to stay around

 optional: cookieObj - HTMLObject::ReadCookie which has the
                       Compressed Cookie already decompressed in it.
           appName - name of Application cookie to check for Session ID in.
           cookieDomain - domain the normal cookie must exist in.
           cookieLife - specifies how long the cookie is to exist for in minutes.
           sessionId - The id generated for the session, if restoring a previous session.

  If sessionId specified then cookieObj and appName must not be
  specified.  This means we are using another Session to store this
  sessions Name -> ID in for use by the SessionHandler class.

  If the sessionId passed in via the cookie existed in the cookie but
  not in the data store, then the errorCode = -1, else for all other 
  errors, errorCode = 0.  This will allow the calling application to decide
  what to do, either blow an error, manually clear the cookie, etc.
  If using sessionId instead of cookie, the sessionId is used without
  doing any look ups other than for existance, etc.

  The value changed in the store is reserved by the module.
  
  sessionId is now available if you don't like referencing session_id.
  
  When you want to indicate that an entry has changed in the store, that
  is deep, update the changed variable.  Your best bet is to change it
  to store the current time value.  Since it defaults to 0, you could
  just change it to 1.
  Ex: $sessionObj->{store}->{changed} = 1;

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( dbiw => undef, dbType => "Postgres", cookieObj => undef, appName => "", cookieDomain => ".pcxperience.com", cookieLife => 30,
               sessionType => "File", fileDir => "", lockDir => "", lockTimeout => "3600", sessionId => "", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{errorCode} = 0;  # make an initial define so AUTOLOADING can work!
  $self->{sessionType} = $args{sessionType};
  $self->{fileDir} = $args{fileDir};
  $self->{lockDir} = $args{lockDir};
  $self->{dbiw} = $args{dbiw};
  $self->{dbType} = $args{dbType};
  $self->{cookieObj} = $args{cookieObj};
  $self->{appName} = $args{appName};
  $self->{cookieDomain} = $args{cookieDomain};
  $self->{cookieLife} = $args{cookieLife};
  $self->{lockTimeout} = $args{lockTimeout};
  $self->{sessionId} = $args{sessionId};
  $self->{useCookie} = (defined $self->{cookieObj} && length $self->{appName} > 0 && length $self->{sessionId} == 0 ? 1 : 0);
  $self->{methods} = Portal::Methods->new(langObj => $self->{langObj});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return $self;
  }
  $self->{variables} = Portal::Data::Variables->new(langObj => $self->{langObj});

  # do validation
  if (!$self->isValid(skip_store => 1))
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  # no error's yet, check for session cookies and instantiate the Apache::Session object.
  my $errorMessage = "Debug: useCookie = '$self->{useCookie}', sessionId = '$self->{sessionId}', appName = '$self->{appName}'";
  my $session_id = "";
  my $dbh = undef;
  if ($self->{sessionType} eq "Database")
  { # protect ourselves from when we call this without passing in the DBIWrapper handle (files only)
    $dbh = $self->{dbiw}->dbh;
  }

  if ($self->{useCookie})
  {
    # look for the cookie
    if (exists $self->{cookieObj}->{cookies}->{$self->{appName}})
    {
      $session_id = $self->{cookieObj}->{cookies}->{$self->{appName}};
    }
  }
  else
  {
    $session_id = $self->{sessionId};
  }

  $errorMessage .= ", sessionId now = '$session_id', sessionType = '$self->{sessionType}'";

  if (length $session_id > 0)
  {
    if ($self->{sessionType} eq "Database")
    {
      # we have to validate the Session Id ourselves as Apache::Session doesn't do a good enough job!
      my $sth = $self->{dbiw}->read(sql => "SELECT id FROM sessions WHERE id = '$session_id'");
      if ($self->{dbiw}->didErrorOccur)
      {
        $self->setError(errorString => "Checking for Session ID = '$session_id' failed!<br>\n" . $self->{dbiw}->errorMessage . $errorMessage);
        return $self;
      }
      # check the result from the database.
      my @result = $sth->fetchrow_array;
      if (scalar @result == 0)
      {
        # The session_id doesn't exist in the Sessions table!  Set a special error.
        $self->{errorCode} = -1;
        $self->setError(errorString => "sessionID = '$session_id' doesn't exist in the store. $errorMessage");
        return $self;
      }
    }
    elsif ($self->{sessionType} eq "File")
    {
      if (! -e "$self->{fileDir}/$session_id")
      {
        # The session_id doesn't exist in the directory!  Set a special error.
        $self->{errorCode} = -1;
        $self->setError(errorString => "sessionID = '$session_id' doesn't exist in the store.");
        return $self;
      }
    }
  }

  # now create the session.
  if ($self->{sessionType} eq "Database")
  {
    # tie to the Apache::Session::dbType database.
    my %hash = ();
    eval "use Apache::Session::$self->{dbType};";
    if ($@)
    {
      $self->setError(errorString => "Eval of 'use Apache::Session::$self->{dbType}' failed!<br>\nError = '$@'.<br>\n$errorMessage");
      return $self;
    }
    if ($self->{dbType} eq "Postgres")
    {
      eval { tie %hash, "Apache::Session::$self->{dbType}", $session_id, { Handle => $dbh, Commit => 1 }; };
    }
    elsif ($self->{dbType} eq "MySQL")
    {
      eval { tie %hash, "Apache::Session::$self->{dbType}", $session_id, { Handle => $dbh, LockHandle => $dbh }; };
    }
    if ($@)
    {
      $self->setError(errorString => "Eval of tie failed!<br>\nError = '$@'.<br>\n$errorMessage");
      return $self;
    }
    $self->{store} = \%hash;
  }
  else
  {
    # tie to the File
    my %hash = ();
    eval "use Apache::Session::File;";
    if ($@)
    {
      $self->setError(errorString => "Eval of 'use Apache::Session::File' failed!<br>\nError = '$@'.<br>\n");
      return $self;
    }

    eval { tie %hash, 'Apache::Session::File', $session_id, { Directory => $self->{fileDir}, LockDirectory => $self->{lockDir} }; };
    if ($@)
    {
      $self->setError(errorString => "Eval of tie failed!<br>\nError = '$@'.<br>\n");
      return $self;
    }
    $self->{store} = \%hash;

    # do any cleanup needed.
    eval "use Apache::Session::Lock::File;";
    if ($@)
    {
      $self->setError(errorString => "Eval of 'use Apache::Session::Lock::File' failed!<br>\nError = '$@'.<br>\n");
      return $self;
    }
    my $l = Apache::Session::Lock::File->new;
    $l->clean($self->{lockDir}, $self->{lockTimeout});
  }

  $self->{store}->{changed} = 0;  # flag to indicate if the session has changed so that we can trigger the update in the backend.
  $self->{session_id} = $self->{store}->{_session_id};
  $self->{sessionId} = $self->{session_id};  # store the new, better named variable.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;
  my %args = ( @_ );
  my $skip_store = ( exists $args{skip_store} ? 1 : 0 );

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->{sessionType} !~ /^(Database|File)$/)
  {
    $self->invalid("sessionType", $self->{sessionType});
  }
  if ($self->{sessionType} eq "Database")
  {
    if (not defined $self->{dbiw})
    {
      $self->missing("dbiw");
    }
    if ($self->{dbType} !~ /^(Postgres|MySQL)$/)
    {
      $self->invalid("dbType", $self->{dbType});
    }
  }
  elsif ($self->{sessionType} eq "File")
  {
    if (length $self->{fileDir} == 0)
    {
      $self->missing("fileDir");
    }
    elsif (! -d $self->{fileDir})
    {
      $self->invalid("fileDir", $self->{fileDir});
    }

    if (length $self->{lockDir} == 0)
    {
      $self->missing("lockDir");
    }
    elsif (! -d $self->{lockDir})
    {
      $self->invalid("lockDir", $self->{lockDir});
    }

    if ($self->{lockTimeout} < 300)
    {
      $self->{lockTimeout} = 300;  # allow lock files to stay around for 5 minutes (minimum).
    }
  }
  if ($self->{useCookie})
  {
    if (not defined $self->{cookieObj})
    {
      $self->missing("cookieObj");
    }
    if (length $self->{appName} == 0)
    {
      $self->missing("appName");
    }
  }
  else
  {
    # currently no validation checks for not working with cookies as it is valid to have an empty sessionId to start out with.
  }
  if (length $self->{cookieDomain} == 0)
  {
    $self->invalid("cookieDomain", $self->{cookieDomain});
  }
  if ($self->{cookieLife} < 30)
  {
    $self->{cookieLife} = 30;  # mandate a minimum of 30 minutes.
  }
  if (!$skip_store)
  {
    if (not defined $self->{store})
    {
      $self->invalid("store", "Session Store is no longer valid!");  ## LANG
    }
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

# delete - Removes the session from the session store
sub delete
{
  my $self = shift;

  if (defined $self->{store})
  {
    tied (%{$self->{store}})->delete if (tied %{$self->{store}});

    $self->{store} = undef;
  }
}

sub DESTROY
{
  my $self = shift;

  if (defined $self->{store})
  {
    untie $self->{store} if (tied %{$self->{store}});  # cleanly close down the connection to the Session.
  }
}

# doc writeCookie - generates the cookie based upon the application name.
# takes:  doc - a HTMLObject document (Base, Normal or FrameSet).
sub writeCookie
{
  my $self = shift;
  my %args = ( doc => undef, @_ );
  my $doc = $args{doc};

  if (not ref($doc))
  {
    $self->setError(errorString => "doc not defined!<br>\n");
    return;
  }

  if (!defined $self->{store})
  {
    $self->setError(errorString => "store not defined!<br>\n");
    return;
  }

  if ($self->{useCookie})
  {
    # write the cookie into the specified document.
    # make the cookie expire X minutes from now in the future (GMT time).
    my $minutes = $self->{cookieLife};
    my $date = $self->{methods}->getDateOffset(date => "now", offset => "$minutes minutes", format => "%a, %d-%b-%Y %H:%M:00 GMT", toTZ => "GMT");
    $doc->setCookie(name => $self->{appName}, value => $self->{session_id}, domain => $self->{cookieDomain}, expires => $date, path => "/");
    if ($doc->didErrorOccur)
    {
      $self->setError($doc->getErrorMessage);
      return;
    }
  }
  $self->{store}->{changed} = 1;  # The application should set this is they need the session to be saved.

  return $doc;
}

1;
__END__

=head1 Exported FUNCTIONS (non-Inline POD)

  void delete(void)
    Deletes the session from the session store.  Technically the Session
    Object is no longer usable.  The store should be undefined.

  void writeCookie(doc)
    writes the cookie into the specified HTMLObject document.

=head1 Exported VARIABLES

  store - the hash that is actually the session.

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Base(3)

=cut
